@@ -1,6 +1,9 @@
 /**
- * See the enclosed file COPYING for license information (GPL). If you
- * did not receive this file, see http://www.horde.org/licenses/gpl.
+ * Provides the javascript for creating a new mailbox.
+ *
+ * @author     Michael Slusarz <slusarz@horde.org>
+ * @copyright  2014 Horde LLC
+ * @license    ASL (http://www.horde.org/licenses/apache)
  */
 
 var IngoNewFolder = {
@@ -20,6 +23,7 @@ var IngoNewFolder = {
             (folder = window.prompt(this.folderprompt + '\n', ''))  &&
             !folder.empty()) {
             this.setNewFolder(elt, folder);
+            e.stop();
         }
     },
 
