@@ -1,6 +1,6 @@
 <?php
 /**
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL).  If you
  * did not receive this file, see http://www.horde.org/licenses/apache.
@@ -12,7 +12,7 @@
 
 /* Determine the base directories. */
 if (!defined('INGO_BASE')) {
-    define('INGO_BASE', __DIR__ . '/..');
+    define('INGO_BASE', realpath(__DIR__ . '/..'));
 }
 
 if (!defined('HORDE_BASE')) {
@@ -21,7 +21,7 @@ if (!defined('HORDE_BASE')) {
     if (file_exists(INGO_BASE . '/config/horde.local.php')) {
         include INGO_BASE . '/config/horde.local.php';
     } else {
-        define('HORDE_BASE', INGO_BASE . '/..');
+        define('HORDE_BASE', realpath(INGO_BASE . '/..'));
     }
 }
 
@@ -49,141 +49,65 @@ class Ingo_Application extends Horde_Registry_Application
 
     /**
      */
-    public $version = 'H5 (3.1.3)';
+    public $version = 'H5 (3.2.0)';
+
+    /**
+     * Cached list of all rulesets.
+     *
+     * @var array
+     */
+    protected $_rulesets;
+
+    /**
+     */
+    protected function _bootstrap()
+    {
+        global $injector;
+
+        $injector->bindFactory('Ingo_Shares', 'Ingo_Factory_Shares', 'create');
+    }
 
     /**
-     * Global variables defined:
-     *   - all_rulesets
-     *   - ingo_shares
      */
     protected function _init()
     {
-        global $injector, $registry, $session;
+        global $registry, $session;
 
         // Create the session.
-        $this->_createSession();
+        if (!$session->exists('ingo', 'script_categories')) {
+            Ingo_Session::create();
+        }
 
         if ($sig = $session->get('ingo', 'personal_share')) {
-            $GLOBALS['ingo_shares'] = $injector->getInstance('Horde_Core_Factory_Share')->create();
-            $GLOBALS['all_rulesets'] = Ingo::listRulesets();
-
             $curr_share = $session->get('ingo', 'current_share');
             $ruleset = Horde_Util::getFormData('ruleset');
 
             /* Select current share. */
             if (is_null($curr_share) || ($ruleset != $curr_share)) {
                 $session->set('ingo', 'current_share', $ruleset);
+                $all_rulesets = $this->_listRulesets();
+
                 if (is_null($curr_share) ||
-                    empty($GLOBALS['all_rulesets'][$ruleset]) ||
-                    !$GLOBALS['all_rulesets'][$ruleset]->hasPermission($registry->getAuth(), Horde_Perms::READ)) {
+                    empty($all_rulesets[$ruleset]) ||
+                    !$all_rulesets[$ruleset]->hasPermission($registry->getAuth(), Horde_Perms::READ)) {
                     $session->set('ingo', 'current_share', $sig);
                 }
             }
-        } else {
-            $GLOBALS['ingo_shares'] = null;
-        }
-    }
-
-    /**
-     * Create an ingo session.
-     *
-     * Session entries:
-     *   - backend: (array) The backend configuration to use.
-     *   - change: (integer) The timestamp of the last time the rules were
-     *                       altered.
-     *   - personal_share: (string) Personal share signature.
-     *   - storage: (array) Used by Ingo_Storage:: for caching data.
-     *   - script_categories: (array) The list of available categories for the
-     *                                Ingo_Script driver in use.
-     *
-     * @throws Ingo_Exception
-     */
-    protected function _createSession()
-    {
-        global $injector, $prefs, $registry, $session;
-
-        if ($session->exists('ingo', 'script_categories')) {
-            return;
-        }
-
-        /* getBackend() and loadIngoScript() will both throw Exceptions, so
-         * do these first as errors are fatal. */
-        foreach (array_filter(Ingo::getBackend()) as $key => $val) {
-            $session->set('ingo', 'backend/' . $key, $val);
-        }
-
-        /* Disable categories as specified in preferences */
-        $locked_prefs = array(
-            'blacklist' => Ingo_Storage::ACTION_BLACKLIST,
-            'forward' => Ingo_Storage::ACTION_FORWARD,
-            'spam' => Ingo_Storage::ACTION_SPAM,
-            'vacation' => Ingo_Storage::ACTION_VACATION,
-            'whitelist' => Ingo_Storage::ACTION_WHITELIST
-        );
-        $locked = array();
-        foreach ($locked_prefs as $key => $val) {
-            if ($prefs->isLocked($key)) {
-                $locked[] = $val;
-            }
-        }
-
-        /* Set the list of categories this driver supports. */
-        $ingo_scripts = $injector->getInstance('Ingo_Factory_Script')
-            ->createAll();
-        $categories = array();
-        foreach ($ingo_scripts as $ingo_script) {
-            $categories = array_merge($categories,
-                                      $ingo_script->availableActions(),
-                                      $ingo_script->availableCategories());
-        }
-        $session->set('ingo', 'script_categories',
-                      array_diff($categories, $locked));
-
-        /* Create shares if necessary. */
-        $factory = $injector->getInstance('Ingo_Factory_Transport');
-        foreach ($session->get('ingo', 'backend/transport', Horde_Session::TYPE_ARRAY) as $transport) {
-            if ($factory->create($transport)->supportShares()) {
-                $shares = $injector->getInstance('Horde_Core_Factory_Share')->create();
-
-                /* If personal share doesn't exist then create it. */
-                $sig = $session->get('ingo', 'backend/id') . ':' . $registry->getAuth();
-                if (!$shares->exists($sig)) {
-                    $identity = $injector->getInstance('Horde_Core_Factory_Identity')->create();
-                    $name = $identity->getValue('fullname');
-                    if (trim($name) == '') {
-                        $name = $registry->getAuth('original');
-                    }
-                    $share = $shares->newShare($registry->getAuth(), $sig, $name);
-                    $shares->addShare($share);
-                }
-
-                $session->set('ingo', 'personal_share', $sig);
-                break;
-            }
         }
     }
 
     /**
      */
-    public function perms()
+    public function getInitialPage()
     {
-        return array(
-            'allow_rules' => array(
-                'title' => _("Allow Rules"),
-                'type' => 'boolean'
-            ),
-            'max_rules' => array(
-                'title' => _("Maximum Number of Rules"),
-                'type' => 'int'
-            )
-        );
+        return strval(Ingo::getInitialPage()->setRaw(true));
     }
 
     /**
      */
     public function menu($menu)
     {
-        global $conf, $ingo_shares, $injector, $prefs, $registry, $session;
+        global $conf, $injector, $prefs, $registry, $session;
 
         $s_categories = $session->get('ingo', 'script_categories');
         $vars = $injector->getInstance('Horde_Variables');
@@ -192,13 +116,13 @@ class Ingo_Application extends Horde_Registry_Application
 
         try {
             if (in_array(Ingo_Storage::ACTION_WHITELIST, $s_categories)) {
-                $menu->add(Horde::url($injector->getInstance('Horde_Registry')->link('mail/showWhitelist')), _("_Whitelist"), 'ingo-whitelist', null, null, null, $vars->page == 'whitelist' ? 'current' : '__noselection');
+                $menu->add(Horde::url($registry->link('mail/showWhitelist')), _("_Whitelist"), 'ingo-whitelist', null, null, null, $vars->page == 'whitelist' ? 'current' : '__noselection');
             }
             if (in_array(Ingo_Storage::ACTION_BLACKLIST, $s_categories)) {
-                $menu->add(Horde::url($injector->getInstance('Horde_Registry')->link('mail/showBlacklist')), _("_Blacklist"), 'ingo-blacklist', null, null, null, $vars->page == 'blacklist' ? 'current' : '__noselection');
+                $menu->add(Horde::url($registry->link('mail/showBlacklist')), _("_Blacklist"), 'ingo-blacklist', null, null, null, $vars->page == 'blacklist' ? 'current' : '__noselection');
             }
         } catch (Horde_Exception $e) {
-            Horde::logMessage($e, 'ERR');
+            Horde::log($e, 'ERR');
         }
 
         if (in_array(Ingo_Storage::ACTION_VACATION, $s_categories)) {
@@ -219,7 +143,8 @@ class Ingo_Application extends Horde_Registry_Application
             $menu->add(Ingo_Basic_Script::url(), _("_Script"), 'ingo-script', null, null, null, $vars->page == 'script' ? 'current' : '__noselection');
         }
 
-        if (!empty($ingo_shares) && empty($conf['share']['no_sharing'])) {
+        if ($injector->getInstance('Ingo_Shares') &&
+            empty($conf['share']['no_sharing'])) {
             $menu->add(
                 '#',
                 _("_Permissions"),
@@ -252,9 +177,8 @@ class Ingo_Application extends Horde_Registry_Application
      */
     public function sidebar($sidebar)
     {
-        global $injector;
+        global $injector, $session;
 
-        $perms = $injector->getInstance('Horde_Core_Perms');
         $actions = array();
         foreach ($injector->getInstance('Ingo_Factory_Script')->createAll() as $script) {
             $actions = array_merge($actions, $script->availableActions());
@@ -264,17 +188,17 @@ class Ingo_Application extends Horde_Registry_Application
             ->retrieve(Ingo_Storage::ACTION_FILTERS)
             ->getFilterList();
 
-        if (!empty($actions) &&
-            ($perms->hasAppPermission('allow_rules') &&
-             ($perms->hasAppPermission('max_rules') === true ||
-              $perms->hasAppPermission('max_rules') > count($filters)))) {
-            $sidebar->addNewButton(_("New Rule"), Ingo_Basic_Rule::url());
+        if (!empty($actions)) {
+            $max = $injector->getInstance('Horde_Core_Perms')->hasAppPermission(Ingo_Perms::getPerm('max_rules'));
+            if (($max === true) || ($max > count($filters))) {
+                $sidebar->addNewButton(_("New Rule"), Ingo_Basic_Rule::url());
+            }
         }
 
-        if (!empty($GLOBALS['ingo_shares']) &&
-            (count($GLOBALS['all_rulesets']) > 1)) {
+        if ($injector->getInstance('Ingo_Shares') &&
+            (count($all_rulesets = $this->_listRulesets()) > 1)) {
             $url = Ingo_Basic_Filters::url();
-            $current = $GLOBALS['session']->get('ingo', 'current_share');
+            $current = $session->get('ingo', 'current_share');
 
             $sidebar->containers['rulesets'] = array(
                 'header' => array(
@@ -283,7 +207,7 @@ class Ingo_Application extends Horde_Registry_Application
                     'collapsed' => false,
                 ),
             );
-            foreach ($GLOBALS['all_rulesets'] as $id => $ruleset) {
+            foreach ($all_rulesets as $id => $ruleset) {
                 $row = array(
                     'selected' => ($current == $id),
                     'url' => $url->add('ruleset', $id),
@@ -295,74 +219,113 @@ class Ingo_Application extends Horde_Registry_Application
         }
     }
 
+    /**
+     */
+    public function perms()
+    {
+        return $GLOBALS['injector']->getInstance('Ingo_Perms')->perms();
+    }
+
     /**
      */
     public function hasPermission($permission, $allowed, $opts = array())
     {
-        if (is_array($allowed)) {
-            switch ($permission) {
-            case 'allow_rules':
-                $allowed = (bool)count(array_filter($allowed));
-                break;
-
-            case 'max_rules':
-                $allowed = max($allowed);
-                break;
-            }
-        }
-        return $allowed;
+        return $GLOBALS['injector']->getInstance('Ingo_Perms')->hasPermission($permission, $allowed, $opts);
     }
 
     /**
      */
     public function removeUserData($user)
     {
+        global $injector;
+
         /* Remove all filters/rules owned by the user. */
         try {
-            $GLOBALS['injector']->getInstance('Ingo_Factory_Storage')->create()->removeUserData($user);
+            $injector->getInstance('Ingo_Factory_Storage')->create()->removeUserData($user);
         } catch (Ingo_Exception $e) {
-            Horde::logMessage($e, 'ERR');
+            Horde::log($e, 'ERR');
             throw $e;
         }
 
         /* Now remove all shares owned by the user. */
-        if (!empty($GLOBALS['ingo_shares'])) {
+        if ($ingo_shares = $injector->getInstance('Ingo_Shares')) {
             /* Get the user's default share. */
             try {
-                $share = $GLOBALS['ingo_shares']->getShare($user);
-                $GLOBALS['ingo_shares']->removeShare($share);
+                $ingo_shares->removeShare($ingo_shares->getShare($user));
             } catch (Horde_Share_Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
                 throw new Ingo_Exception($e);
             }
 
             /* Get a list of all shares this user has perms to and remove the
              * perms. */
             try {
-                $shares = $GLOBALS['ingo_shares']->listShares($user);
+                $shares = $ingo_shares->listShares($user);
                 foreach ($shares as $share) {
                     $share->removeUser($user);
                 }
             } catch (Horde_Share_Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
             }
 
             /* Get a list of all shares this user owns and has perms to delete
              * and remove them. */
             try {
-                $shares = $GLOBALS['ingo_shares']->listShares($user, array(
+                $shares = $ingo_shares->listShares($user, array(
                     'attributes' => $user,
                     'perm' => Horde_Perms::DELETE
                 ));
             } catch (Horde_Share_Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
                 throw new Ingo_Exception($e);
             }
 
             foreach ($shares as $share) {
-                $GLOBALS['ingo_shares']->removeShare($share);
+                $ingo_shares->removeShare($share);
             }
         }
     }
 
+    /**
+     * Returns all rulesets a user has access to.
+     *
+     * @return array  The ruleset list.
+     */
+    protected function _listRulesets()
+    {
+        global $injector, $registry;
+
+        if (isset($this->_rulesets)) {
+            return $this->_rulesets;
+        }
+
+        $this->_rulesets = array();
+
+        try {
+            if (!($share = $injector->getInstance('Ingo_Shares'))) {
+                return $this->_rulesets;
+            }
+
+            $tmp = $share->listShares(
+                $registry->getAuth(),
+                array('perm' => Horde_Perms::SHOW)
+            );
+        } catch (Horde_Share_Exception $e) {
+            Horde::log($e, 'ERR');
+            return $this->_rulesets;
+        }
+
+        /* Check if filter backend of the share still exists. */
+        $backends = Ingo::loadBackends();
+
+        foreach ($tmp as $id => $ruleset) {
+            list($backend) = explode(':', $id);
+            if (isset($backends[$backend])) {
+                $this->_rulesets[$id] = $ruleset;
+            }
+        }
+
+        return $this->_rulesets;
+    }
+
 }
