@@ -1,6 +1,6 @@
 <?php
 /**
- * Copyright 2003-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2003-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL).  If you
  * did not receive this file, see http://www.horde.org/licenses/apache.
@@ -218,9 +218,15 @@ class Ingo_Script_Sieve extends Ingo_Script_Base
         if (empty($folder)) {
             $action[] = new Ingo_Script_Sieve_Action_Discard();
         } elseif ($folder == Ingo::BLACKLIST_MARKER) {
-            $action[] = new Ingo_Script_Sieve_Action_Addflag(array('flags' => Ingo_Storage::FLAG_DELETED));
+            $action[] = new Ingo_Script_Sieve_Action_Addflag(array(
+                'flags' => Ingo_Storage::FLAG_DELETED,
+                'imapflags' => !empty($this->_params['imapflags'])
+            ));
             $action[] = new Ingo_Script_Sieve_Action_Keep();
-            $action[] = new Ingo_Script_Sieve_Action_Removeflag(array('flags' => Ingo_Storage::FLAG_DELETED));
+            $action[] = new Ingo_Script_Sieve_Action_Removeflag(array(
+                'flags' => Ingo_Storage::FLAG_DELETED,
+                'imapflags' => !empty($this->_params['imapflags'])
+            ));
         } else {
             $action[] = new Ingo_Script_Sieve_Action_Fileinto(array_merge($this->_params, array('folder' => $folder)));
         }
@@ -335,8 +341,11 @@ class Ingo_Script_Sieve extends Ingo_Script_Base
             $mime_headers = new Horde_Mime_Headers();
             $headers = $mime_headers->listHeaders();
             $headers['Mailing-List'] = null;
-            $tmp = new Ingo_Script_Sieve_Test_Exists(array('headers' => implode("\n", array_keys($headers))));
-            $tests[] = new Ingo_Script_Sieve_Test_Not($tmp);
+            foreach (array_keys($headers) as $h) {
+                $tests[] = new Ingo_Script_Sieve_Test_Not(
+                    new Ingo_Script_Sieve_Test_Exists(array('headers' => $h))
+                );
+            }
             $vals = array('headers' => 'Precedence',
                           'match-type' => ':is',
                           'strings' => "list\nbulk\njunk",
@@ -483,13 +492,19 @@ class Ingo_Script_Sieve extends Ingo_Script_Base
             switch ($filter['action']) {
             case Ingo_Storage::ACTION_KEEP:
                 if (!empty($filter['flags'])) {
-                    $action[] = new Ingo_Script_Sieve_Action_Addflag(array('flags' => $filter['flags']));
+                    $action[] = new Ingo_Script_Sieve_Action_Addflag(array(
+                        'flags' => $filter['flags'],
+                        'imapflags' => !empty($this->_params['imapflags'])
+                    ));
                 }
 
                 $action[] = new Ingo_Script_Sieve_Action_Keep();
 
                 if (!empty($filter['flags'])) {
-                    $action[] = new Ingo_Script_Sieve_Action_Removeflag(array('flags' => $filter['flags']));
+                    $action[] = new Ingo_Script_Sieve_Action_Removeflag(array(
+                        'flags' => $filter['flags'],
+                        'imapflags' => !empty($this->_params['imapflags'])
+                    ));
                 }
                 break;
 
@@ -499,14 +514,20 @@ class Ingo_Script_Sieve extends Ingo_Script_Base
 
             case Ingo_Storage::ACTION_MOVE:
                 if (!empty($filter['flags'])) {
-                    $action[] = new Ingo_Script_Sieve_Action_Addflag(array('flags' => $filter['flags']));
+                    $action[] = new Ingo_Script_Sieve_Action_Addflag(array(
+                        'flags' => $filter['flags'],
+                        'imapflags' => !empty($this->_params['imapflags'])
+                    ));
                 }
 
                 $action[] = new Ingo_Script_Sieve_Action_Fileinto(array_merge($this->_params, array('folder' => $filter['action-value'])));
 
                 if (!empty($filter['flags'])) {
-                    $action[] = new Ingo_Script_Sieve_Action_Removeflag(array('flags' => $filter['flags']));
-                    }
+                    $action[] = new Ingo_Script_Sieve_Action_Removeflag(array(
+                        'flags' => $filter['flags'],
+                        'imapflags' => !empty($this->_params['imapflags'])
+                    ));
+                }
                 break;
 
             case Ingo_Storage::ACTION_REJECT:
@@ -522,7 +543,10 @@ class Ingo_Script_Sieve extends Ingo_Script_Base
 
             case Ingo_Storage::ACTION_REDIRECTKEEP:
                 if (!empty($filter['flags'])) {
-                    $action[] = new Ingo_Script_Sieve_Action_Addflag(array('flags' => $filter['flags']));
+                    $action[] = new Ingo_Script_Sieve_Action_Addflag(array(
+                        'flags' => $filter['flags'],
+                        'imapflags' => !empty($this->_params['imapflags'])
+                    ));
                 }
 
                 $parser = new Horde_Mail_Rfc822();
@@ -533,31 +557,47 @@ class Ingo_Script_Sieve extends Ingo_Script_Base
                 $action[] = new Ingo_Script_Sieve_Action_Keep();
 
                 if (!empty($filter['flags'])) {
-                    $action[] = new Ingo_Script_Sieve_Action_Removeflag(array('flags' => $filter['flags']));
+                    $action[] = new Ingo_Script_Sieve_Action_Removeflag(array(
+                        'flags' => $filter['flags'],
+                        'imapflags' => !empty($this->_params['imapflags'])
+                    ));
                 }
                 break;
 
             case Ingo_Storage::ACTION_MOVEKEEP:
                 if (!empty($filter['flags'])) {
-                    $action[] = new Ingo_Script_Sieve_Action_Addflag(array('flags' => $filter['flags']));
+                    $action[] = new Ingo_Script_Sieve_Action_Addflag(array(
+                        'flags' => $filter['flags'],
+                        'imapflags' => !empty($this->_params['imapflags'])
+                    ));
                 }
 
                 $action[] = new Ingo_Script_Sieve_Action_Keep();
                 $action[] = new Ingo_Script_Sieve_Action_Fileinto(array_merge($this->_params, array('folder' => $filter['action-value'])));
 
                 if (!empty($filter['flags'])) {
-                    $action[] = new Ingo_Script_Sieve_Action_Removeflag(array('flags' => $filter['flags']));
+                    $action[] = new Ingo_Script_Sieve_Action_Removeflag(array(
+                        'flags' => $filter['flags'],
+                        'imapflags' => !empty($this->_params['imapflags'])
+                    ));
                 }
                 break;
 
             case Ingo_Storage::ACTION_FLAGONLY:
                 if (!empty($filter['flags'])) {
-                    $action[] = new Ingo_Script_Sieve_Action_Addflag(array('flags' => $filter['flags']));
+                    $action[] = new Ingo_Script_Sieve_Action_Addflag(array(
+                        'flags' => $filter['flags'],
+                        'imapflags' => !empty($this->_params['imapflags'])
+                    ));
                 }
                 break;
 
             case Ingo_Storage::ACTION_NOTIFY:
-                $action[] = new Ingo_Script_Sieve_Action_Notify(array('address' => $filter['action-value'], 'name' => $filter['name']));
+                $action[] = new Ingo_Script_Sieve_Action_Notify(array(
+                    'address' => $filter['action-value'],
+                    'name' => $filter['name'],
+                    'notify' => !empty($this->_params['notify'])
+                ));
                 break;
 
             case Ingo_Storage::ACTION_WHITELIST:
