@@ -1,6 +1,6 @@
 <?php
 /**
- * Copyright 2002-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file LICENSE for license information (ASL).  If you
  * did not receive this file, see http://www.horde.org/licenses/apache.
@@ -129,42 +129,45 @@ class Ingo_Storage
      */
     public function store($ob)
     {
-        $type = $ob->obType();
-        if (in_array($type, array(self::ACTION_BLACKLIST,
-                                  self::ACTION_VACATION,
-                                  self::ACTION_WHITELIST,
-                                  self::ACTION_FORWARD,
-                                  self::ACTION_SPAM))) {
-            $filters = $this->retrieve(self::ACTION_FILTERS);
-            if ($filters->findRuleId($type) === null) {
-                switch ($type) {
-                case self::ACTION_BLACKLIST:
-                    $name = 'Blacklist';
-                    break;
+        global $session;
 
-                case self::ACTION_VACATION:
-                    $name = 'Vacation';
-                    break;
+        switch ($type = $ob->obType()) {
+        case self::ACTION_BLACKLIST:
+            $name = 'Blacklist';
+            break;
 
-                case self::ACTION_WHITELIST:
-                    $name = 'Whitelist';
-                    break;
+        case self::ACTION_VACATION:
+            $name = 'Vacation';
+            break;
 
-                case self::ACTION_FORWARD:
-                    $name = 'Forward';
-                    break;
+        case self::ACTION_WHITELIST:
+            $name = 'Whitelist';
+            break;
 
-                case self::ACTION_SPAM:
-                    $name = 'Spam Filter';
-                    break;
-                }
-                $filters->addRule(array('action' => $type, 'name' => $name));
-                $this->store($filters);
-            }
+        case self::ACTION_FORWARD:
+            $name = 'Forward';
+            break;
+
+        case self::ACTION_SPAM:
+            $name = 'Spam Filter';
+            break;
+
+        default:
+            $name = null;
+            break;
+        }
+
+        if (!is_null($name) &&
+            ($filters = $this->retrieve(self::ACTION_FILTERS)) &&
+            ($filters->findRuleId($type) === null)) {
+            $filters->addRule(array('action' => $type, 'name' => $name));
+            $this->store($filters);
         }
 
         $this->_store($ob);
-        $this->_cache[$ob->obType()] = $ob;
+        $this->_cache[$type] = $ob;
+
+        $session->set('ingo', 'change', time());
     }
 
     /**
@@ -319,4 +322,45 @@ class Ingo_Storage
 	    throw new Ingo_Exception(_("Removing user data is not supported with the current filter storage backend."));
     }
 
+    /**
+     * Output description for a rule.
+     *
+     * @param array $rule  Rule.
+     *
+     * @return string  Text description.
+     */
+    public function ruleDescription($rule)
+    {
+        $condition_size = count($rule['conditions']) - 1;
+        $descrip = '';
+
+        foreach ($rule['conditions'] as $key => $val) {
+            $info = $this->getTestInfo($val['match']);
+            $descrip .= sprintf("%s %s \"%s\"", _($val['field']), $info->label, $val['value']);
+
+            if (!empty($val['case'])) {
+                $descrip .= ' [' . _("Case Sensitive") . ']';
+            }
+
+            if ($key < $condition_size) {
+                $descrip .= ($rule['combine'] == self::COMBINE_ALL)
+                    ? _(" and")
+                    : _(" or");
+                $descrip .= "\n  ";
+            }
+        }
+
+        $descrip .= "\n" . $this->getActionInfo($rule['action'])->label;
+
+        if ($rule['action-value']) {
+            $descrip .= ': ' . $rule['action-value'];
+        }
+
+        if ($rule['stop']) {
+            $descrip .= "\n[stop]";
+        }
+
+        return $descrip;
+    }
+
 }
